"""Library to communicate with AWS cost explorer API"""
import re

class CE:
    """cost explorer class based on boto client"""

    no_spotinstance_filter = {
        "Not": {"Dimensions": {"Key": "PURCHASE_TYPE", "Values": ["Spot Instances"]}}
    }

    no_tag_key_stack_filter = {"Tags": {"Key": "Stack", "MatchOptions": ["ABSENT"]}}

    def __init__(self, session, start_time, end_time, granularity):
        self.ce_client = session.client("ce")
        self.timeperiod = {"Start": start_time, "End": end_time}
        self.granularity = granularity

    def get_cost_usage(self, groupby, filters=None):
        """Calls get_cost_and_usage boto method"""
        if filters:
            response = self.ce_client.get_cost_and_usage(
                TimePeriod=self.timeperiod,
                Granularity=self.granularity,
                Metrics=["NET_AMORTIZED_COST"],
                Filter=filters,
                GroupBy=groupby,
            )
        else:
            response = self.ce_client.get_cost_and_usage(
                TimePeriod=self.timeperiod,
                Granularity=self.granularity,
                Metrics=["NET_AMORTIZED_COST"],
                GroupBy=groupby,
            )
        # logger.log(f"get_cost_and_usage response:\n {response}")
        return response

    def get_cost_notagkey_stack_groupby_service(self):
        """Gets service wise cost break down in an account"""
        # Fetch Service Wise Cost
        services_costs = {}
        service_groupby = [{"Type": "DIMENSION", "Key": "SERVICE"}]
        cost_usage_service_response = self.get_cost_usage(
            service_groupby, CE.no_tag_key_stack_filter
        )
        service_cost_with_no_stack_tag = cost_usage_service_response['ResultsByTime'][0]['Groups']
        for response in service_cost_with_no_stack_tag:
            if "Metrics" in response and "NetAmortizedCost" in response["Metrics"]:
                service = response["Keys"][0]
                cost = round(float(response["Metrics"]["NetAmortizedCost"]["Amount"]), 2)
                if cost > 0:
                    services_costs[service] = cost
        top_n_services = min(len(services_costs), 7)
        sorted_costs = dict(sorted(services_costs.items(), key=lambda item: item[1], reverse=True))
        top_services = dict(list(sorted_costs.items())[:top_n_services])
        # Sum the costs of the remaining services
        others_cost = round(sum(list(sorted_costs.values())[top_n_services:]), 2)
        # Combine the top 5 with the "Others" category
        summary_costs = top_services
        if others_cost > 0:
            summary_costs['Others'] = others_cost
        return sorted_costs, summary_costs

    def get_cost_by_stack(self):
        """Gets service wise cost break down in an account"""
        stacks_costs = {}
        groupby = [{"Type": "TAG", "Key": "Stack"}]
        cost_usage_response = self.get_cost_usage(groupby)
        time_period = cost_usage_response["ResultsByTime"][0]["TimePeriod"]
        total_stacks_data = cost_usage_response["ResultsByTime"][0]["Groups"]
        for response in total_stacks_data:
            if "Metrics" in response and "NetAmortizedCost" in response["Metrics"]:
                stack = self.get_stack_name(response["Keys"][0])
                cost = round(float(response["Metrics"]["NetAmortizedCost"]["Amount"]), 2)
                if (
                    cost > 0 and stack != "NoTagKey:Stack"
                ):  # NoTagkey:Stack services grouping is done separately
                    stacks_costs[stack] = cost
        return time_period, stacks_costs

    def get_stack_name(self, metric_stack_name):
        """returns name of the stack, if no stack name found it indicates inefficient tagging"""

        pattern = r"Stack\$(.*)"
        match = re.search(pattern, metric_stack_name)
        name = "NoTagKey:Stack"
        if match and match.group(1):
            name = match.group(1)
        return name
