"""Helper functions for EKS"""
import boto3
import botocore.exceptions
from botocore.exceptions import ClientError

from ieg_cloud_scan.utils.logger import logger


class EKS:
    """Class representing EKS"""

    def list_eks_clusters_across_regions(self):
        """Function returning all EKS clusters in the account"""

        supported_regions = boto3.Session().get_available_regions("eks")
        eks_clusters = []

        for region in supported_regions:
            eks_client = boto3.client("eks", region_name=region)
            try:
                response = eks_client.list_clusters()
                clusters = response["clusters"]
                eks_clusters.extend(clusters)
            except botocore.exceptions.ClientError as client_error:
                if (
                    client_error.response["Error"]["Code"]
                    == "UnrecognizedClientException"
                ):
                    logger.log_error(f"UnrecognizedClientException in {region}")
                if (
                    client_error.response["Error"]["Code"] == "AccessDenied"
                    and "Explicit deny" in client_error.response["Error"]["Message"]
                ):
                    # Handle SCP explicit deny error
                    logger.log_error(
                        f"Access denied due to Service Control Policy (SCP) explicit deny in {region}"
                    )
                else:
                    # Handle other errors
                    logger.log_error(f"An error occurred:, {client_error}")

        return eks_clusters

    def delete_eks_cluster(self, region_clusters):
        """Function to delete the list of EKS clusters"""

        for region, clusters in region_clusters:
            eks_client = boto3.client("eks", region_name=region)
            for cluster in clusters:
                # List and delete nodegroups before deleting the cluster
                nodegroups = eks_client.list_nodegroups(clusterName=cluster)[
                    "nodegroups"
                ]
                for nodegroup_name in nodegroups:
                    logger.log(f"Deleting nodegroup {nodegroup_name}...")
                    eks_client.delete_nodegroup(
                        clusterName=cluster, nodegroupName=nodegroup_name
                    )

                # Delete the EKS cluster
                logger.log(f"Deleting cluster {cluster}...")
                eks_client.delete_cluster(name=cluster)

                logger.log(f"Cluster {cluster} deletion initiated.")
