"""AWS Medialive operations"""
import re

import boto3

# from tkinter import NO
from botocore.exceptions import EndpointConnectionError

from ieg_cloud_scan.utils.logger import logger


class MediaLive:
    """Helper methods to get infor about medialive channels and inputs"""
    def __init__(self, region) -> None:
        try:
            self.medialive_client = boto3.client("medialive", region_name=region)
            self.channels_response = self.medialive_client.list_channels()
            self.inputs_response = self.medialive_client.list_inputs()
        except EndpointConnectionError as exception:
            # Handle the exception as per your requirement
            logger.log_error(f"Error getting medialive info for {region} Exception:{exception}")
            self.medialive_client = None
            self.channels_response = None
            self.inputs_response = None


    def get_medialive_instances_without_tag(self, tag_key, tag_value_pattern):
        """Get list of medialive instance without tag_key and tag_value pattern match combination"""
        untagged_medialive = {}

        if self.inputs_response is None or self.channels_response is None:
            return untagged_medialive

        # Get all MediaLive inputs
        inputs = self.inputs_response["Inputs"]

        # Get all MediaLive channels
        channels = self.channels_response["Channels"]

        untagged_inputs = {}
        untagged_channels = {}

        # Check inputs for the specified tag key
        for input_data in inputs:
            input_state = input_data["State"]
            tags = input_data.get("Tags", {})

            untagged_inputs = {
                input_id: input_name
                for input_id, input_name in input_data.items()
                if input_state == "ATTACHED"
                and (
                    tag_key not in tags
                    or not re.match(tag_value_pattern, tags[tag_key])
                )
            }

        # Check channels for the specified tag key and a pattern value
        for channel_data in channels:
            channel_state = channel_data["State"]
            tags = channel_data.get("Tags", {})

            untagged_channels = {
                channel_id: channel_name
                for channel_id, channel_name in channel_data.items()
                if channel_state == "RUNNING"
                and (
                    tag_key not in tags
                    or not re.match(tag_value_pattern, tags[tag_key])
                )
            }
        logger.log(f"Untagged channels {len(untagged_channels.keys())}")
        logger.log(f"Untagged inputs {len(untagged_inputs.keys())}")
        untagged_medialive.update(
            {"untagged_inputs": untagged_inputs} if untagged_inputs else {}
        )
        untagged_medialive.update(
            {"untagged_channels": untagged_channels} if untagged_channels else {}
        )

        return untagged_medialive

    def get_output_info_of_channels(self):
        """Get list of medialive instance without tag_key and tag_value pattern match combination"""   
        
        if self.inputs_response is None or self.channels_response is None:
            return None

        # Get all MediaLive channels
        channels = self.channels_response["Channels"]
        amgid_destinations = {}
        for channel_data in channels:
            tags = channel_data.get("Tags", {})
            channel_state = channel_data["State"]
            if (
                channel_state == "RUNNING"
                and "AMGID" in tags
                and channel_data["Destinations"]
            ):
                amgid = tags["AMGID"]
                destinations_count = amgid_destinations.get(amgid, 0)
                amgid_destinations[amgid] = destinations_count + len(
                    channel_data["Destinations"]
                )

        return amgid_destinations
