import boto3
from botocore.exceptions import ClientError
from ieg_cloud_scan.utils.logger import logger
from datetime import datetime, timedelta

class RDS:
    def __init__(self, region):
        self.rds_client = boto3.client("rds", region_name=region)
        self.response = self.rds_client.describe_db_instances()

    def get_untagged_rds_instances(self, tag_key):
        untagged_rds = {}
        untagged_tagkey_instances = {}
        for instance in self.response["DBInstances"]:
            instance_arn = instance["DBInstanceArn"]
            instance_name = instance["DBInstanceIdentifier"]

            if "TagList" in instance:
                tags = {tag["Key"]: tag["Value"] for tag in instance["TagList"]}
                if tag_key not in tags:
                    logger.log(f"{instance_arn} not tagged")
                    untagged_tagkey_instances.update({instance_arn: instance_name})
            else:
                untagged_tagkey_instances.update({instance_arn: instance_name})
        untagged_rds = (
            {"untagged_rds": untagged_tagkey_instances}
            if untagged_tagkey_instances
            else {}
        )
        logger.log(f"Untagged RDS {len(untagged_tagkey_instances.keys())}")

        return untagged_rds

    def rds_with_mutuallyexclusive_tagvalues(
        self, rds_arns, tag_name_value_1, tag_name_value_2
    ):
        tag1, value1 = next(iter(tag_name_value_1.items()))
        tag2, value2 = next(iter(tag_name_value_2.items()))
        # Create a dictionary to store the filtered volumes
        filtered_rds = {}
        # Filter volumes without the tag_name tag and attached to on-demand instances
        for volume in self.ondemand_volumes:
            tags = {tag["Key"]: tag["Value"] for tag in volume["Tags"]}
            if tag1 in tags and tag2 in tags:
                # a generalised function has been written to catch kubernetes.io/created-for/pvc/namespace: monitoring to not be tagged with AMGID:AMGX0999
                namespace = tags.get(tag1)
                amgid = tags.get(tag2)
                if namespace != value1 and amgid == value2:
                    filtered_rds[volume["VolumeId"]] = self.get_volume_name(tags)
        return filtered_rds
    
    
    def __get_cloudwatch_metric_data(self, db_cluster_identifier, metric_name, start_time, end_time):
        try:
            response = self.cloudwatch.get_metric_data(
                MetricDataQueries=[
                    {
                        'Id': 'm1',
                        'MetricStat': {
                            'Metric': {
                                'Namespace': 'AWS/RDS',
                                'MetricName': metric_name,
                                'Dimensions': [
                                    {
                                        'Name': 'DBClusterIdentifier',
                                        'Value': db_cluster_identifier
                                    },
                                ]
                            },
                            'Period': 300,  # 5 min in seconds
                            'Stat': 'Average',
                        },
                    },
                ],
                StartTime=start_time,
                EndTime=end_time,
            )
        except Exception as e:
            print(f"Error in getting cloudwatch metric for {db_cluster_identifier} in {self.region} : {e}" , flush=True )
            return -1
        else:
            if 'Values' in response['MetricDataResults'][0]:
                return max( [0] + response['MetricDataResults'][0]['Values']) #response can be empty
            else:
                return -1


    def stop_rds_cluster_if_no_connections(self, cluster ):
        db_cluster_identifier = cluster['DBClusterIdentifier']

        tag_autostop= "not_found"
        for tags in cluster['TagList']:
            if tags['Key']=="autostop":
                tag_autostop= tags['Value']

        if tag_autostop=="not_found":
            tag_autostop= "true"
            try:
                self.rds_client.add_tags_to_resource(
                    ResourceName= cluster['DBClusterArn'],
                    Tags=[
                        {
                            'Key': 'autostop',
                            'Value': tag_autostop
                        },
                    ]
                )
                print(f"Added 'autostop:true' tag to RDS cluster {db_cluster_identifier} in {self.region}.", flush=True)
            except Exception as e:
                print(f"Error adding tag to RDS cluster {db_cluster_identifier} in {self.region}: {e}", )

        if tag_autostop!='true':
            print( f"could not stop {db_cluster_identifier} in {self.region}. autostop:false tag is found" )
            return

        cluster_status= cluster['Status']
        if cluster_status!= 'available':
            print( f"could not stop {db_cluster_identifier} in {self.region}. cluster is not available" )
            return


        metric_name = 'DatabaseConnections' 

        end_time = datetime.utcnow()
        start_time = end_time - timedelta(days=7)

        max_connection_count = self.__get_cloudwatch_metric_data(db_cluster_identifier, metric_name, start_time, end_time)

        if max_connection_count == 0:
            try:
                self.rds_client.stop_db_cluster(DBClusterIdentifier=db_cluster_identifier)
                print(f"RDS cluster {db_cluster_identifier} in {self.region} stopped successfully.", flush=True )
            except Exception as e:
                print(f"Error stopping RDS cluster {db_cluster_identifier} in {self.region}: {e}")
        else:
            print(f"Connections found for {db_cluster_identifier} in {self.region}, RDS cluster will not be stopped.", flush=True)