import boto3
import datetime
import time
import uuid


class Redis:

    def __init__(self):
        self.redis_client = None
        self.cloudwatch = None
        self.region = None

    def __get_cloudwatch_metric_data(self, node_id, metric_name, start_time, end_time):
        try:
            response = self.cloudwatch.get_metric_data(
                MetricDataQueries=[
                    {
                        'Id': 'm1',
                        'MetricStat': {
                            'Metric': {
                                'Namespace': 'AWS/ElastiCache',
                                'MetricName': metric_name,
                                'Dimensions': [
                                    {
                                        'Name': 'CacheClusterId',
                                        'Value': node_id
                                    },
                                ]
                            },
                            'Period': 300,  # 5 minutes
                            'Stat': 'Average',
                        },
                        'ReturnData': True,
                    },
                ],
                StartTime=start_time,
                EndTime=end_time,
            )
        except Exception as e:
            print(f"Error getting CloudWatch metric data for node {node_id} in {self.region} : {e}")
            return 1
        else:
            if 'Values' in response['MetricDataResults'][0]:
                return max([0] + response['MetricDataResults'][0]['Values'])
            else:
                return 1
            
    def __wait_for_available_state(self, cluster_id):
        max_wait_time = 50
        wait_interval = 10
        elapsed_time = 0

        while elapsed_time < max_wait_time:
            try:
                response = self.redis_client.describe_replication_groups(
                    ReplicationGroupId=cluster_id
                )
                status = response['ReplicationGroups'][0]['Status']
                if status == 'available':
                    return True
            except Exception as e:
                print(f"Error waiting for cluster {cluster_id} in {self.region} to be in an available state")
            finally:
                time.sleep(wait_interval)
                elapsed_time += wait_interval

        print(f"Timed out waiting for cluster {cluster_id} in {self.region} to be in an available state.")
        return False
    
    def __delete_redis_cluster_if_unused(self, cluster):
        cluster_id = cluster['ReplicationGroupId']
        metric_name = 'KeyBasedCmds'
        end_time = datetime.datetime.utcnow()
        start_time = end_time - datetime.timedelta(days=14)

        # Check if the cluster is blacklisted
        autostop_tag= None
        unused_tag= None

        if not self.__wait_for_available_state(cluster_id):
            print("cluster is not in available state. skipping...")
            return

        tags = self.redis_client.list_tags_for_resource(ResourceName=cluster['ARN'])
        autostop_tag = next((tag['Value'] for tag in tags.get('TagList', []) if tag['Key'] == 'autostop'), None)
        unused_tag = next((tag['Value'] for tag in tags.get('TagList', []) if tag['Key'] == 'unused'), None)

        if autostop_tag is not None and autostop_tag.lower() == 'false':
            print(f"Skipping cluster {cluster_id} in {self.region} because it is blacklisted (autostop:false)")
            return

        elif autostop_tag != "true":
            try:
                self.redis_client.add_tags_to_resource(
                    ResourceName=cluster['ARN'],
                    Tags=[
                        {
                            'Key': 'autostop',
                            'Value': 'true'
                        }
                    ]
                )
            except Exception as e:
                print(f"failed adding autostop tag to {cluster_id} in {self.region}")
        
        max_metric_data = -1
        primary_nodes = []

        for ng in cluster.get('NodeGroups', []):
            for n in ng.get('NodeGroupMembers', []):
                node_id = n.get('CacheClusterId', "")
                if n.get('CurrentRole', '') == 'primary':
                    primary_nodes.append(node_id)
                max_metric_data = max(max_metric_data, self.__get_cloudwatch_metric_data(node_id, metric_name, start_time, end_time))

        if len(primary_nodes) > 0 and max_metric_data == 0:
            # self.deleted_redis_cluster_count += 1
            print(f'{cluster_id} in {self.region} is unused')

            if unused_tag != "true":
                try:
                    if not self.__wait_for_available_state(cluster_id):
                        print("cluster is not in available state. skipping...")
                        return
                    self.redis_client.add_tags_to_resource(
                        ResourceName=cluster['ARN'],
                        Tags=[
                            {
                                'Key': 'unused',
                                'Value': 'true'
                            }
                        ]
                    )
                    if not self.__wait_for_available_state(cluster_id):
                        print("cluster is not in available state. skipping...")
                        return
                    for primary_node in primary_nodes:
                        backup_name = f'daily-scan-{primary_node}-{uuid.uuid4().hex}'
                        self.redis_client.create_snapshot(
                            ReplicationGroupId=cluster_id,
                            CacheClusterId=primary_node,
                            SnapshotName=backup_name,
                            Tags=[
                                {
                                    'Key': 'created_by',
                                    'Value': 'md.ansari@amagi.com'
                                }
                            ]
                        )
                        print(f'Backup {backup_name} created for node {primary_node} in {self.region}')

                except Exception as e:
                    print( f"Error while adding unused tag for cluster {cluster_id} in {self.region}")
                    return
                else:
                    print( f"{cluster_id} in {self.region} is marked to be deleted tomorrom if left unused" )
            else:
                self.redis_client.delete_replication_group(
                    ReplicationGroupId=cluster_id,
                )
                print(f'Cluster {cluster_id} deleted with backups in {self.region}')
    

    def run(self):
        regions = boto3.Session().get_available_regions('elasticache')
        # regions = ['ap-south-1']

        for region in regions:
            self.region = region
            try:
                self.redis_client = boto3.client('elasticache', region_name=region)
                self.cloudwatch = boto3.client('cloudwatch', region_name=region)
                response = self.redis_client.describe_replication_groups()
                clusters = response['ReplicationGroups']
            except Exception as e:
                print(f"Could not describe all the clusters in {region}: {e}")
            else:
                for cluster in clusters:
                    self.__delete_redis_cluster_if_unused(cluster)

        # self.__generate_report()
        