"""Egotist does cost break up analysis using boto cost explorer"""

import argparse
import base64
import os
import shutil
from datetime import datetime, timedelta

from ieg_cloud_scan.aws.ce import CE
from ieg_cloud_scan.aws.secretmanager import SecretManager
from ieg_cloud_scan.aws.sts import STS
from ieg_cloud_scan.helpers.costinfo import StackCostAnalyser
from ieg_cloud_scan.helpers.gitinfo import GithubCPAppAnalyser
from ieg_cloud_scan.utils.chart_gen import plot_piechart
from ieg_cloud_scan.utils.github import Github
from ieg_cloud_scan.utils.logger import logger
from ieg_cloud_scan.utils.mailer import Mailer

from ieg_cloud_scan.config.constants import CloudMappingDict

cwd = os.getcwd()
cost_breakdown_txt = f"{cwd}/cost_breakdown.txt"


def get_timeperiod_granularity(span, start_date, end_date):
    """returns time range and granularity to be passed to cost explorer api"""
    now_time = datetime.utcnow()
    # end_time = (now_time - timedelta(hours=24)).strftime("%Y-%m-%d")
    end_time = now_time.strftime("%Y-%m-%d")
    granularity = "DAILY"
    cost_limit = 250
    if span == "weekly":
        start_time = (now_time - timedelta(hours=168)).strftime("%Y-%m-%d")
        granularity = "MONTHLY"
        cost_limit = cost_limit * 7
    elif span == "monthly":
        start_time = (now_time - timedelta(hours=672)).strftime("%Y-%m-%d")
        granularity = "MONTHLY"
        cost_limit = cost_limit * 7 * 4
    elif span == "custom":
        end_time_dt = datetime.strptime(end_date, "%Y-%m-%d")
        start_time_dt = datetime.strptime(start_date, "%Y-%m-%d")
        cost_limit = cost_limit * (end_time_dt - start_time_dt).days
        start_time = str(start_date)
        end_time = str(end_date)
        granularity = "MONTHLY" if end_time_dt - start_time_dt > timedelta(days=1) else "DAILY"
    else:
        start_time = (now_time - timedelta(hours=24)).strftime("%Y-%m-%d")
    time_period = {"start_time": start_time, "end_time": end_time}
    logger.log(f"Running egotists with time period: {time_period} and granularity: {granularity}")
    return time_period, cost_limit, granularity


def get_cost_by_dept_category(
    cloud_account,
    all_stacks_data,
    service_cost,
    all_stacks_gitops_data,
    response_time_period,
):
    """slices and dices CE response by comparing against egotist_config.py"""
    stack_analyser = StackCostAnalyser(
        cloud_account, all_stacks_data, all_stacks_gitops_data, cost_breakdown_txt
    )
    cost_by_dept_category = stack_analyser.get_stackcost_by_dept()
    stack_analyser.write_dept_stack_costs(cost_by_dept_category, service_cost, response_time_period)
    return cost_by_dept_category


def get_all_stacks_gitops_data(branch):
    """slices and dices github response by comparing against egotist_config.py"""
    cp_app_instances = {}
    ghapp_secrets = {
        'ieg-scan_ghapp_app_id': '',
        'ieg-scan_ghapp_installation_id': '',
        'ieg-scan_ghapp_pvt_key': '',
    }
    sm_gh_dict = SecretManager("ap-south-1").get_secrets('ieg-scan/ghapp', ghapp_secrets)
    app_id = sm_gh_dict['ieg-scan_ghapp_app_id']
    installation_id = sm_gh_dict['ieg-scan_ghapp_installation_id']
    pvt_key = base64.b64decode(sm_gh_dict['ieg-scan_ghapp_pvt_key'])
    gh = Github(app_id, installation_id, pvt_key)
    git_branch_repo = [
        {'branch': branch, 'repo': 'amagimedia/gitops-cloudport-argocd-dev-deploy'},
        {'branch': 'main', 'repo': 'amagimedia/argocd_deploy'},
    ]
    for config in git_branch_repo:
        branch_name = config['branch']
        repo_url = config['repo']
        dest_dir = f"{cwd}/gitops/{branch_name}"
        gh.clone(repo_url, branch_name, dest_dir)
        app_analyser = GithubCPAppAnalyser(dest_dir)
        instances = app_analyser.get_all_cp_instances()
        cp_app_instances[repo_url] = instances
    # once all the apps are analysed, its better to remove the cloned directories
    shutil.rmtree(f"{cwd}/gitops/", ignore_errors=True)
    return cp_app_instances


def prepare_mail_content(
    branch,
    cost_by_dept_category,
    sorted_service_cost,
    response_time_period,
    interval,
    cost_limit,
    to_list,
):
    """prepares body and attachments to be sent in analysis mail"""

    chart_data = cost_by_dept_category["Departments"]
    unaccounted_cost = chart_data.get("Unaccounted", {}).get("cost", 0)
    # no_tag_key_data = (
    #     chart_data.get("Unaccounted", {}).get("categories", {}).get("None", {}).get("stacks", {})
    # )
    # no_tag_key_cost = no_tag_key_data.get("NoTagKey:Stack", 0)
    no_tag_key_cost = round(sum(value for value in sorted_service_cost.values()), 2)

    tagged_stacks_cost = cost_by_dept_category["cost"]
    total_cost = tagged_stacks_cost + no_tag_key_cost
    start_time = response_time_period["Start"]
    end_time = response_time_period["End"]

    body = "n=CurrentUTCtime at invocation; TimeRange n-48 to n-24\n\n"
    body += f"Start:{start_time} End:{end_time} (but not including {end_time})\n\n"
    body += f"Total cost:{total_cost}\n\n"
    body += f"Stacks cost not mapped to Dept:Category: {unaccounted_cost}\n"
    body += f"Services with NoTagKey:Stack: {no_tag_key_cost}\n"
    for service, cost in sorted_service_cost.items():
        body += f"\t\t Service:{service} Cost:{cost}\n"

    # Pie chart to plot cost by department
    chart_title = f"Total:$ {total_cost} From:{start_time} \
        End:{end_time} \n\n Unaccounted cost: {unaccounted_cost}\n\n"
    if "Unaccounted" in chart_data:
        del chart_data["Unaccounted"]

    images = []
    if len(chart_data.keys()):
        images.append(f"{cwd}/CostByDept.png")
        plot_piechart(chart_title, chart_data, images[0])

    # Pie chart to plot categorywise breakup of department with cost > daily_limit
    chart_data_cost_limit = {k: v for k, v in chart_data.items() if v["cost"] > cost_limit}
    body += f"\n\nDepartments exceeding ${cost_limit}\n"
    for dept, dept_info in chart_data_cost_limit.items():
        body += f"\t{dept} : {dept_info['cost']}\n"
        chart_title = f"Breakdown of {dept} Cost:{dept_info['cost']} From:{start_time} \
            End:{end_time} \n\n"
        image_name = f"{cwd}/{dept}.png"
        images.append(image_name)
        plot_piechart(chart_title, dept_info["categories"], image_name)

    subject = f" {interval} {branch} Cost analysis from {start_time} to {end_time}(not including)"
    content = {
        "subject": subject,
        "body": body,
        "text_attachment": cost_breakdown_txt,
        "image_attachment": images,
        "recepients": to_list,
    }
    logger.log(f"EmailContent:\n{content}")
    return content

def get_session(cloud_name: str):
    """ fetches secret from AWS Secret Manager and sets the corresponding account's session """
    cloud_account_id = CloudMappingDict[cloud_name]
    secret_content = SecretManager(region="us-east-1").get_secrets(secret_name="egotist_cross_account_secret")
    for role_arn, external_id in secret_content.items():
        if cloud_account_id in role_arn:
            return STS().assume_iam_role(role_arn=role_arn, external_id=external_id)

    return STS().default_profile()

def parse_args():
    '''Parses cli passed by the user'''
    parser = argparse.ArgumentParser(description="Egotist command line tool")
    parser.add_argument(
        "--span",
        choices=["weekly", "monthly", "custom"],
        nargs="?",
        help="Command to execute",
    )
    parser.add_argument(
        "--cloud", choices=CloudMappingDict.keys(), help="name of the cloudport AWS account"
    )
    parser.add_argument("--to_list", help="email ids separated by ,")
    parser.add_argument("--start_date", nargs="?", help="Start date for custom command")
    parser.add_argument("--end_date", nargs="?", help="End date for custom command")

    cli_args = parser.parse_args()
    if cli_args.span == "custom" and (cli_args.start_date is None or cli_args.end_date is None):
        parser.error("Custom command requires start_date and end_date")
    return cli_args


def run():
    """function invoked by poetry"""
    cli_args = parse_args()
    timeperiod, cost_limit, interval = get_timeperiod_granularity(
        cli_args.span, cli_args.start_date, cli_args.end_date
    )
    cloud_account = cli_args.cloud

    session = get_session(cloud_name=cloud_account)

    # GET COST EXPLORER DATA
    ce = CE(
        session=session, start_time=timeperiod["start_time"], end_time=timeperiod["end_time"], granularity=interval
    )

    # queries cost explorer and gets all untagged services costs
    all_service_cost, sorted_all_service_cost = ce.get_cost_notagkey_stack_groupby_service()

    # queries cost explorer and gets all tagged stacks costs
    response_time_period, all_stacks_data = ce.get_cost_by_stack()



    # queries GH argocd repos and gets all application information about clusters
    all_stacks_gitops_data = get_all_stacks_gitops_data(f"aws-{cloud_account}")

    # groups GH, cost data into respective department(e.g qa) and category info(e.g regression)
    cost_by_dept_category = get_cost_by_dept_category(
        cloud_account,
        all_stacks_data,
        all_service_cost,
        all_stacks_gitops_data,
        response_time_period,
    )

    logger.log(f"Time period:{response_time_period}\n\n")
    logger.log(cost_by_dept_category)
    logger.log(f"All services cost:\n {all_service_cost}")

    email_content = prepare_mail_content(
        cloud_account,
        cost_by_dept_category,
        sorted_all_service_cost,
        response_time_period,
        interval,
        cost_limit,
        cli_args.to_list,
    )
    Mailer("us-east-1").send_email_with_retries(email_content)
