"""Helper to group stack cost into department and category"""

from collections import defaultdict

import ieg_cloud_scan.config.egotist_config as egotist


class StackCostAnalyser:
    """Helper methods to group stack cost into department and category"""

    def __init__(self, cloud_account, all_stack_info, all_gitops_info, output_file) -> None:
        self.cloud_account = cloud_account
        self.all_stack_info = all_stack_info
        self.output_file = output_file
        self.all_gitops_info = all_gitops_info

    def find_dept_by_stack_name(self, dept_stack, stack_name):
        """returns unkown if not found in config/egotist_config.py"""

        for department, categories in dept_stack.items():
            for cat_name, stack_list in categories.items():
                if stack_name in stack_list:
                    return department, cat_name
        return "Unaccounted", "None"

    def get_cp_apps_info(self, stackname):
        """returns the app level information of the stack"""
        appdetails = {}
        for repo in self.all_gitops_info.keys():
            if stackname in self.all_gitops_info[repo]:
                appdetails = self.all_gitops_info[repo][stackname]
        return appdetails

    def get_stackcost_by_dept(self):
        """Groups stacks and their cost to department, category"""
        dept_stack_costs = {"Departments": defaultdict(default_dept_entry), "cost": 0}
        total_cost = 0

        for stack_name, stack_cost in self.all_stack_info.items():
            dept, category = self.find_dept_by_stack_name(
                egotist.dept_stack[self.cloud_account], stack_name
            )
            total_cost = round(total_cost + stack_cost, 2)

            dept_info = dept_stack_costs["Departments"][dept]
            category_info = dept_info["categories"][category]
            stack_info = category_info["stacks"][stack_name]

            # Update costs
            category_info["cost"] = round(category_info["cost"] + stack_cost, 2)
            dept_info["cost"] = round(dept_info["cost"] + stack_cost, 2)
            stack_info["cost"] = stack_cost  # Assuming stack_cost is already rounded
            stack_info["apps"] = self.get_cp_apps_info(stack_name)
        dept_stack_costs["cost"] = total_cost
        # logger.log(dept_stack_costs)
        return dept_stack_costs

    def write_dept_stack_costs(self, dept_stack_costs, service_cost, date_range):
        """function to print the dept/category/stack cost info"""
        # Sort the dictionary by cost
        sorted_dept_stack_costs = self.sort_dict_by_cost(dept_stack_costs["Departments"])
        untagged_service_cost = round(sum(value for value in service_cost.values()), 2)
        with open(self.output_file, "w", encoding="utf-8") as file:
            file.write(f"Date Range: {date_range} \n")
            file.write(f"Total cost: {dept_stack_costs['cost'] + untagged_service_cost} \n")
            file.write(f"Tagged Stack costs: {dept_stack_costs['cost']} \n")
            file.write(f"Untagged services costs: {untagged_service_cost} \n")
            file.write("\tUntagged service wise cost breakdown:\n")
            for service, cost in service_cost.items():
                file.write(f"\t\tService: {service} Cost: ${cost}\n")
            file.write("\n--------------------------------------------------------------------\n")
            for dept, dept_data in sorted_dept_stack_costs.items():
                file.write(f"Department: {dept} Cost: ${dept_data.get('cost', 0)}\n")
                file.write("\tCategory-wise cost breakdown:\n")
                sorted_categories = self.sort_dict_by_cost(dept_data.get("categories", {}))
                for category, category_data in sorted_categories.items():
                    file.write(f"\t\tCategory: {category} Cost: ${category_data.get('cost', 0)}\n")

                    file.write("\t\t\tStack-wise cost breakdown:\n")
                    sorted_stacks = self.sort_dict_by_cost(category_data.get("stacks", {}))
                    for stack, stack_info in sorted_stacks.items():
                        if stack_info.get('cost', 0) != 0.0:
                            file.write(
                                f"\t\t\t\tStack: {stack} Cost: ${stack_info.get('cost',0)} Apps: {stack_info.get('apps',None)}\n"
                            )
                            # if stack == "NoTagKey:Stack":
                            #     for service, cost in service_cost.items():
                            #         file.write(f"\t\t\t\t\tService: {service} Cost: ${cost}\n")
                    file.write(
                        "\n--------------------------------------------------------------------\n"
                    )

    def get_cost(self, item):
        """helper function to get cost item"""
        # Check if item is a dictionary (department or category)
        if isinstance(item, dict):
            return item.get("cost", 0)
        # If item is an integer (cost value for a stack)
        return item

    def sort_dict_by_cost(self, data):
        """sort each data item by cost"""
        # Sorting the dictionary based on the cost in descending order
        return dict(sorted(data.items(), key=lambda item: self.get_cost(item[1]), reverse=True))


def default_stack_entry():
    """Initialises stacks with default cost 0 and apps = {}"""
    return {"cost": 0, "apps": {}}


def default_category_entry():
    """Initialises Category with default cost 0 and stacks = {}"""
    return {"cost": 0, "stacks": defaultdict(default_stack_entry)}


def default_dept_entry():
    """Initialises Department with default cost 0 and categories = {}"""
    return {"cost": 0, "categories": defaultdict(default_category_entry)}
