"""Helper class to categorise gitops info of cloudport"""

import os

import yaml


class GithubCPAppAnalyser:
    """Helper class to categorise gitops info of cloudport"""

    def __init__(self, dest_directory) -> None:
        self.dest_directory = dest_directory
        self.valid_instances = {}

    def get_all_cp_instances(self):
        """Function to loop through all CP apps"""
        app_configs = [
            {'app_name': 'cp-live', 'num_parts': 4},
            {'app_name': 'cp-automation', 'num_parts': 3},
            {'app_name': 'cp-playout', 'num_parts': 4},
        ]
        # List all dev stages from the dir where git clone was done
        dev_stages = next(os.walk(self.dest_directory))[1]
        for dev_stage in dev_stages:
            deployment_path = os.path.join(self.dest_directory, f"{dev_stage}/deployment")
            for config in app_configs:
                self.generic_get_cp_instances(
                    config['app_name'], config['num_parts'], deployment_path
                )
        return self.valid_instances

    def initialize_key(self, dct, key, init_type):
        """simple function to check key exists, avoids code duplication"""
        if key not in dct:
            dct[key] = init_type()

    def initialize_valid_instances(self, stack_name, app_name):
        """initialises the dictionary which holds all CP app instances"""
        if stack_name not in self.valid_instances:
            self.valid_instances[stack_name] = {}
        if app_name not in self.valid_instances[stack_name]:
            if app_name == 'cp-playout':
                self.valid_instances[stack_name][app_name] = {}
            else:
                self.valid_instances[stack_name][app_name] = set()

    def generic_get_cp_instances(self, app_name, num_parts, deployment_path):
        """Function to form a dictionary of CP apps and their instances across deployments"""
        app_path = f"{deployment_path}/{app_name}"
        # logger.log(f"Analysing {app_path}")

        for dirpath, _, filenames in os.walk(app_path):
            # relpath:
            # aws-iota/dev/deployment/cp-automation
            rel_path = os.path.relpath(dirpath, app_path)
            # parts: aws-aps1 iota-conn bucky02-automation
            parts = rel_path.split(os.sep)

            # if automation parts=3, if live/playout parts=4 ; parse till Chart.yaml is found
            if len(parts) == num_parts and "Chart.yaml" in filenames:
                region, cluster = parts[:2]
                # stack_name = aws-use1-iota-snlr1
                stack_name = f"{region}-{cluster}"

                # using app_name as key to group the application instances
                # e.g valid_instances['aws-use1-iota-snlr1']['cp-playout']
                self.initialize_valid_instances(stack_name, app_name)

                # player instances lie one level further down compared to live/automation
                # e.g sinclairdemo1-636-001-dd , sinclairdemo1-636-001-player is under
                # dev/deployment/cp-playout/aws-use1/iota-prime/sinclairdemo1-playout
                # where as sinclairdemo1-live, /sinclairdemo1-automation is under
                # dev/deployment/cp-automation/aws-use1/iota-prime/
                # dev/deployment/cp-live/aws-use1/iota-prime/
                if app_name in ['cp-automation', 'cp-live']:
                    self.valid_instances[stack_name][app_name].add(
                        parts[-1 if app_name == 'cp-automation' else -2]
                    )

                elif app_name == 'cp-playout':
                    # sinclairdemo1-playout , sinclairdemo1-636-001-dd
                    appln, appln_type = parts[2:4]
                    self.initialize_key(self.valid_instances[stack_name][app_name], appln, dict)
                    # to find the type of component we need to get pod type from override_values.yml
                    overrides_yml_path = (
                        f"{app_path}/{region}/{cluster}/{appln}/{appln_type}/override_values.yaml"
                    )
                    # podtype = dd, player, switcher
                    podtype = self.get_appln_podtype(overrides_yml_path)
                    self.initialize_key(
                        self.valid_instances[stack_name][app_name][appln], podtype, list
                    )
                    self.valid_instances[stack_name][app_name][appln][podtype].append(appln_type)
        return self.valid_instances

    def get_appln_podtype(self, overrides_yml_path):
        """gets podType: from playout/type/override_values.yaml"""
        try:
            podtype = "player"  # we default to player
            if os.path.isfile(overrides_yml_path):
                with open(overrides_yml_path, 'r', encoding="utf-8") as file:
                    over_rides = yaml.safe_load(file)
                    podtype = over_rides['cp-playout']['podType']
            return podtype
        except Exception as e:
            return "misconfiguration"
