"""Class and method to send mails using SES"""

import os
import time
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

import boto3

from ieg_cloud_scan.utils.logger import logger


class Mailer:
    """Class to send mails using SES"""

    def __init__(self, region) -> None:
        # Create an SES client
        self.ses_client = boto3.client("ses", region_name=region)

    def send_email_with_image(self, mail_content):
        """Create a new message with multipart/related as the MIME type"""
        msg = MIMEMultipart("related")
        msg["Subject"] = mail_content["subject"]
        msg["From"] = "karthik.k@amagi.com"
        msg["To"] = ", ".join(mail_content["recepients"].split(","))

        # Create a text part for the email body
        text_part = MIMEText(mail_content["body"])
        msg.attach(text_part)

        # Read the image file and create an image part
        for image in mail_content["image_attachment"]:
            with open(image, "rb") as file:
                img_data = file.read()
                image_name = os.path.basename(image)
                image_part = MIMEImage(img_data, name=image_name)
                image_part.add_header("Content-ID", image_name)
                msg.attach(image_part)

        with open(mail_content["text_attachment"], "r", encoding="utf-8") as file_handle:
            file_content = file_handle.read()
        text_content = MIMEText(file_content, "plain")
        text_content.add_header(
            "Content-Disposition",
            "attachment",
            filename=os.path.basename(mail_content["text_attachment"]),
        )
        msg.attach(text_content)
        try:
            self.ses_client.send_raw_email(RawMessage={"Data": msg.as_string()})
        except Exception as exception:
            logger.log_error(f"An error occurred: {exception}")
            return False
        logger.log("Email sent successfully!")
        return True

    def send_email_with_retries(self, email_content):
        """Call sendmail with retries and delays between retries"""
        max_retries = 5
        retry_delay = 15  # time delay between retries in seconds
        for i in range(max_retries):
            if Mailer("ap-south-1").send_email_with_image(email_content):
                break
            logger.log(f"Retrying {i + 1} of {max_retries}...")
            time.sleep(retry_delay)
        else:
            logger.log_error("Failed to send the email after max retries")
